/* CAP - Console Access Protection                               */
/* This provides physical security for your console. When you    */
/* run this program, it will clear the screen and prompt for     */ 
/* a password. After so many failed attempts it will lock the    */
/* tty and not allow them to try anymore. While this program     */
/* is running, they can't abort this program, and they can not   */
/* switch consoles either. The only only way around this is to   */
/* reboot the computer, in which case you will obviously know    */
/* know someone tried to access your console. This will log the  */
/* date and time the user tried to get access into your console. */
/*								 */
/*                     Shok (Matt Conover)                       */
/*              shok@sekurity.org, shok@janova.org               */

#include <pwd.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <syslog.h>
#include <linux/vt.h>
#include <sys/stat.h>
#include <sys/types.h>

#define ERROR -1
#define SAME   0
#define LOCKOUT 3 /* How long in minutes to lock out of the console.     */
#define MAXFAIL 3 /* Number of times they can enter an invalid password. */
                  /* before being locked out for LOCKOUT minutes.        */

char *strip(char *str); /* Used to strip newlines from ctime().          */

void main()
{
  int fd;
  int uid;
  int failed    = 0; /* Number of failed attempts out of MAXFAIL.      */
  int totfailed = 0; /* Number of total failed attempts (not reseted). */
  
  time_t tm;

  char *p;
  char thetime[64];
  char salt[2], salt1, salt2;
  char *pass, passwd[128], realpasswd[128];
  struct passwd *pwd;

  if ((fd = open("/dev/tty", O_RDWR)) == ERROR) {
	perror("opening console (/dev/tty)");
	exit(ERROR);
  }
  
  /* Disable signals (so attackers can't abort program). */
  signal(SIGHUP,  SIG_IGN);
  signal(SIGINT,  SIG_IGN);
  signal(SIGTSTP, SIG_IGN);
  signal(SIGTERM, SIG_IGN);
  signal(SIGQUIT, SIG_IGN);

  printf("w00w00!\n");
  printf("Now locking console.\n"
         "Password to unlock the console will be set to your password.\n");
  sleep(5);

  if ((ioctl(fd, VT_LOCKSWITCH, 1)) == ERROR) {
     perror("locking console (/dev/tty/)");
     exit(ERROR);
  }

  uid = getuid();
  pwd = getpwuid(uid);
  strncpy(realpasswd, pwd->pw_passwd, sizeof(realpasswd));

  /* Get salts from the encrypted password. */
  p = realpasswd;
  sprintf(salt, "%c%c", p[0], p[1]);

  system("clear");
  printf("Console is now locked. Hit enter for a prompt...");

  getchar();

  /* Used to log invalid password attempts. */
  openlog("conprot", LOG_CONS, LOG_AUTHPRIV); 

  while (1) {
     /* Get the password from the user. */
     pass = (char *)getpass("Enter password: ");

     strncpy(passwd, pass, sizeof(passwd));

     /* Encrypt the password from getpass(). */
     strncpy(passwd, (char *)crypt(pass, salt), sizeof(passwd));

     if ((strcmp(passwd, realpasswd)) == SAME) {
        /* Unlock the console, to allow it to switch. */
        if ((ioctl(fd, VT_UNLOCKSWITCH, 1)) == ERROR) {
           perror("unlocking console (/dev/tty/)");
           exit(ERROR);
        }

        closelog(); /* Close logging. */

	system("clear");

        printf("Everything is now restored.\n");

        if (totfailed == 0) printf("No one tried to access the console.\n");
        else printf("Total number of failed attempts to unlock console: %d\n",
	            totfailed);

        exit(0);
     } else {
	failed++, totfailed++; /* Increase number of failed attempts. */

        /* Log bad attempts to syslog. */
        tm = time(NULL);

        snprintf(thetime, sizeof(thetime), (char *)ctime(&tm));
        strip(thetime); /* Strip new lines out of the time. */
        syslog(LOG_WARNING, "Failed access attempt on: %s", thetime);

        printf("Invalid password.\n");

	if (failed >= MAXFAIL) {
	   printf("Maximum number of failed attempts.\n"
                  "Now locking for %d minutes.\n", LOCKOUT);

	   sleep(LOCKOUT * 60); /* Convert the minutes to seconds. */
	   failed = 0; /* Reset the number of failed attempts.     */
        }
     }
  }
}

char *strip(char *str)
{
  register int i;

  for (i = 0; str[i]; i++) 
      /* Strip newline out of string. */
      /* We do this because syslog appends the newline itself. */
      if (str[i] == '\n') str[i] = '\0'; 

  return str;
}
